@@ -65,8 +65,16 @@ class ItemCollection(TypedDict, total=False):
     """STAC Item Collection."""
 
     type: str
-    stac_version: str
-    stac_extensions: Optional[List[str]]
     features: List[Item]
     links: List[Dict[str, Any]]
     context: Optional[Dict[str, int]]
+
+
+class Collections(TypedDict, total=False):
+    """All collections endpoint.
+
+    https://github.com/radiantearth/stac-api-spec/tree/master/collections
+    """
+
+    collections: List[Collection]
+    links: List[Dict[str, Any]]
